package de.cdiag.jugdo.arquillian;

import java.util.List;

import javax.ejb.EJB;

import javax.ejb.EJBException;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.junit.InSequence;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

import de.cdiag.jugdo.arquillian.bl.CustomerService;
import de.cdiag.jugdo.arquillian.domain.Customer;

/**
 * Implementation of the first test case of the presentation.
 * 
 * @author Sebastian Lammering, CDI AG (www.cdi-ag.de)
 * 
 */
@RunWith(Arquillian.class)
public class CustomerServiceTest {

	/**
	 * Create the archive to deploy in the EJB container. The archive contains
	 * all classes of the source folder and the file "test-persistence.xml" as
	 * "persistence.xml" to configure the persistence unit.
	 * 
	 * @return archive to deploy.
	 */
	@Deployment
	public static Archive<?> createArchive() {
		JavaArchive archive = ShrinkWrap.create(JavaArchive.class);
		archive.addPackages(true, CustomerService.class.getPackage());
		archive.addClass(Customer.class);
		archive.addAsManifestResource("test-persistence.xml", "persistence.xml");
		return archive;
	}

	/** Service, injected by arquillian. */
	@EJB
	CustomerService customerService;

	/** First customer for the optimistic locking test case. */
	static Customer firstCustomer;

	/** Save a customer */
	@Test
	@InSequence(1) // to execute the test methods in a defined order
	public void save() {

		Customer customer = new Customer("JugDo", "JavaUserGroup Dortmund");

		Customer savedCustomer = customerService.save(customer);

		Assert.assertNotNull(savedCustomer.getId());

		List<Customer> customers = customerService.getAll();
		Assert.assertEquals(1, customers.size());

	}

	/** Load a customer and save the customer in the static field. */
	@Test
	@InSequence(2)
	public void loadFirst() {
		firstCustomer = customerService.getByKey("JugDo");
		Assert.assertNotNull(firstCustomer);
	}

	/** Load, change and save the customer. */
	@Test
	@InSequence(3)
	public void loadSecondAndSave() {
		Customer loadedCustomer = customerService.getByKey("JugDo");
		loadedCustomer.setName("42");
		customerService.save(loadedCustomer);
	}

	/** Save the first loaded customer. An {@link EJBException} (wrapped an OptimisticLockingException) is expected. */
	@Test(expected = EJBException.class)
	@InSequence(4)
	public void saveSecond() {
		customerService.save(firstCustomer);
	}

}
